local dt = require "darktable"

local mods = {}

mods.colorbalancergb = "iop/colorbalancergb"
mods.color_calibration = "iop/channelmixerrgb"
mods.diffuse_or_sharpen = "iop/diffuse"
mods.denoise_profiled = "iop/denoiseprofile"
mods.exposure = "iop/exposure"
mods.filmicrgb = "iop/filmicrgb"
mods.highlight_reconstruction = "iop/highlights"

local function toggle(mod, item, instance, delay)
  local inst = instance or 0
  local wait = delay or 500
  local status = dt.gui.action(mod .. "/" .. item, inst, "", "", 1.0)
  dt.print_log("toggled " .. mod .. " " .. item .. " with status " .. status)
  dt.control.sleep(wait)
  local status = dt.gui.action(mod .. "/" .. item, inst, "", "", 1.0)
  dt.print_log("toggled " .. mod .. " " .. item .. " with status " .. status)
end

local function enable(mod, instance)
  local inst = instance or 0
  dt.gui.action(mod, inst, "enable", "on", 1.0)
end

local function disable(mod, instance)
  local inst = instance or 0
  dt.gui.action(mod, inst, "enable", "off", 1.0)
end

local function reset(mod, instance)
  local inst = instance or 0
  dt.gui.action(mod, inst, "reset", "activate", 1.0)
end

local function select(mod, item, instance, value)
  local inst = instance or 0
  dt.gui.action(mod .. "/" .. item, inst, "selection", value, 1.0)
end

local function abs_adjust(mod, item, instance, amount)
  local inst = instance or 0
  dt.gui.action(mod .. "/" .. item, inst, "value", "set", amount)
end

local function rel_adjust(mod, item, instance, amount)
  local inst = instance or 0
  local status = dt.gui.action(mod .. "/" .. item, inst, "value", "set", "")
  dt.print_log("status is " .. status .. " and adjusted value should be " .. amount + status)
  dt.gui.action(mod .. "/" .. item, inst, "value", "set", status + amount)
end

local function button(mod, item, instance)
  local inst = instance or 0
  dt.gui.action(mod .. "/" .. item, inst, "", "", 1.0)
end

local function action(mod, item, instance, amount)
  local inst = instance or 0
  dt.gui.action(mod .. "/" .. item, inst, "", "", amount)
end

local styles = {
  assess_image = {
    {op = toggle, mod = "global", item = "toggle focus peaking", inst = 0, delay = 1500},
    {op = toggle, mod = "views", item = "darkroom/color assessment", inst = 0, delay = 1500},
    {op = toggle, mod = "views", item = "darkroom/raw overexposed/- toggle", inst = 0, delay = 1500},
    {op = toggle, mod = "views", item = "darkroom/overexposed/- toggle", inst = 0, delay = 1500},
    {op = toggle, mod = "views", item = "darkroom/gamut check", inst = 0, 1500},
    {op = toggle, mod = "views", item = "darkroom/guide lines/- toggle", inst = 0, delay = 1500}
  },
  auto_straighten = {
    {op = button, mod = "iop/ashift", item = "fit/vertical", inst = 0},
    {op = button, mod = "iop/ashift", item = "structure/lines", inst = 0}
  },
  blown_highlights = {
    {op = select, mod = mods.filmicrgb, item = "preserve chrominance", inst = 0, value = "item:luminance Y"},
    {op = select, mod = mods.highlight_reconstruction, item = "method", inst = 0, value = "item:segmentation based"}
  },
  enable_final_modules = {
    {op = enable, mod = mods.diffuse_or_sharpen, inst = 0},
    {op = enable, mod = mods.colorbalancergb, inst = 0},
    {op = enable, mod = mods.diffuse_or_sharpen, inst = 1},
  }
}

dt.register_event("autolevel", "shortcut",
  function(event, shortcut) 
    dt.gui.action("iop/filmicrgb/auto tune levels", 0, "", "", 1.0)
    dt.print_log("turned filmic auto tune levels on")
    dt.control.sleep(500)
    dt.gui.action("iop/filmicrgb/auto tune levels", 0, "", "", 1.0)
    dt.print_log("turned filmic auto tune levels off")
    status = dt.gui.action("iop/filmicrgb/preserve chrominance", 0, "selection", "", "")
    dt.print_log("status is " .. status)
    dt.gui.action("iop/filmicrgb/preserve chrominance", 0, "selection", "next", 1.0)
    status = dt.gui.action("iop/filmicrgb/preserve chrominance", 0, "selection", "item: doesnt exist", 1.0)
    dt.print_log("status is " .. status)
    status = dt.gui.action("iop/filmicrgb", 0, "presets", "", "")
    dt.print_log("status is " .. status)
    status = dt.gui.action("iop/filmicrgb", 0, "presets", "item: default", 1.0)
    dt.print_log("status is " .. status)
    for i = 0,3 do
      status = dt.gui.action("iop/filmicrgb", i, "", "", "")
      dt.print_log("status is " .. status)
    end
    status = dt.gui.action("iop/filmicrgb", 0, "reset", "", 0)
    dt.print_log("status is " .. status)
    status = dt.gui.action("iop/filmicrgb/black relative exposure", 0, "value", "set", -8.0)
    dt.print_log("status is " .. status)
    dt.gui.action("iop/diffuse", 0, "enable", "on", 1.0)
    dt.gui.action("processing modules/diffuse or sharpen/presets/add local contrast", 0, "", "", 1.0)
  end, ("filmic autotune")
)

dt.register_event("assess_image", "shortcut",
  function(event, shortcut)
    toggle("global", "toggle focus peaking", 0, 1500)
    dt.control.sleep(1000)
    toggle("views", "darkroom/color assessment", 0, 1500)
    dt.control.sleep(1000)
    toggle("views", "darkroom/raw overexposed/toggle", 0, 1500)
    dt.control.sleep(1000)
    toggle("views", "darkroom/overexposed/toggle", 0, 1500)
    dt.control.sleep(1000)
    toggle("views", "darkroom/gamut check", 0, 1500)
    dt.control.sleep(1000)
    toggle("views", "darkroom/guide lines/toggle", 0, 1500)
  end , "assess image"
)

dt.register_event("auto_straighten", "shortcut",
  function(event, shortcut)
    dt.gui.action("iop/ashift", 0, "focus", "on", 1.0)
    dt.control.sleep(500)
    button("iop/ashift", "structure/auto", 0)
    dt.control.sleep(1500)
    dt.gui.action("iop/ashift/fit/vertical", 0, "", "", 1.0)
    dt.control.sleep(500)
    dt.gui.action("iop/ashift", 0, "focus", "off", 1.0)
    --button("iop/ashift", "structure/auto", 0)
  end , "auto straighten"
)

dt.register_event("blown_highlights", "shortcut",
  function(event, shortcut)
    select(mods.filmicrgb, "preserve chrominance", 0, "item:luminance Y")
    select(mods.highlight_reconstruction, "method", 0, "item:segmentation based")
  end , "blown highlights"
)

local function enable_final_modules()
  local img = dt.gui.views.darkroom.display_image()
  local cnt = 0
  local iso = img.exif_iso
  dt.print_log("iso is " .. iso)
  local function first_last()
    enable(mods.diffuse_or_sharpen, 3)
    enable(mods.diffuse_or_sharpen, 4)
  end
  enable(mods.colorbalancergb, 0)
  enable("iop/cacorrectrgb", 0)
  if iso < 1000 then
    cnt = 2
  elseif iso < 6400 then
    cnt = 3
  elseif iso < 10000 then
    cnt = 4
  else
    cnt = 5
  end

  if cnt == 2 then
    first_last()
  elseif cnt == 3 then
    enable(mods.denoise_profiled, 0)
    reset(mods.denoise_profiled, 0)
    enable(mods.diffuse_or_sharpen, 3)
    enable(mods.diffuse_or_sharpen, 4)
  elseif cnt == 4 then
    enable(mods.denoise_profiled, 0)
    enable(mods.diffuse_or_sharpen, 1)
    enable(mods.diffuse_or_sharpen, 3)
    enable(mods.diffuse_or_sharpen, 4)
  else
    enable(mods.denoise_profiled, 0)
    abs_adjust(mods.denoise_profiled, "strength", 0, 2.0)
    enable(mods.diffuse_or_sharpen, 1)
    enable(mods.diffuse_or_sharpen, 2)    
    enable(mods.diffuse_or_sharpen, 3)    
    enable(mods.diffuse_or_sharpen, 4)    
  end
--  for i = 1, cnt do
--    enable(mods.diffuse_or_sharpen, i - 1)
--  end
end

dt.register_event("enable_final_modules", "shortcut",
  function(event, shortcut)
    enable_final_modules()
  end , "enable final modules"
)

dt.register_event("enable_final_modules1", "shortcut",
  function(event, shortcut)
    enable_final_modules(2)
  end , "enable final modules1"
)

dt.register_event("enable_final_modules2", "shortcut",
  function(event, shortcut)
    enable_final_modules(3)
  end , "enable final modules2"
)

dt.register_event("enable_final_modules3", "shortcut",
  function(event, shortcut)
    enable_final_modules(4)
  end , "enable final modules3"
)

dt.register_event("enable_final_modules4", "shortcut",
  function(event, shortcut)
    enable_final_modules(5)
  end , "enable final modules4"
)

dt.register_event("enable_final_modules_denoise", "shortcut",
  function(event, shortcut)
    enable_final_modules(3)
  end , "enable final modules denoise"
)

dt.register_event("enable_final_modules_denoise_medium", "shortcut",
  function(event, shortcut)
    enable_final_modules(4)
  end , "enable final modules denoise medium"
)

dt.register_event("enable_final_modules_denoise_coarse", "shortcut",
  function(event, shortcut)
    enable_final_modules(5)
  end , "enable final modules denoise coarse"
)

dt.register_event("add_quarter_stop_exposure", "shortcut",
  function(event, shortcut)
    rel_adjust(mods.exposure, "exposure", 0, 0.25)
  end , "exposure plus quarter"
)

dt.register_event("add_half_stop_exposure", "shortcut",
  function(event, shortcut)
    rel_adjust(mods.exposure, "exposure", 0, 0.5)
  end , "exposure plus half"
)

dt.register_event("subtract_quarter_stop_exposure", "shortcut",
  function(event, shortcut)
    rel_adjust(mods.exposure, "exposure", 0, -0.25)
  end , "exposure minus quarter"
)

dt.register_event("subtract_half_stop_exposure", "shortcut",
  function(event, shortcut)
    rel_adjust(mods.exposure, "exposure", 0, -0.5)
  end , "exposure minus half"
)

dt.register_event("add_quarter_strength_denoise", "shortcut",
  function(event, shortcut)
    rel_adjust(mods.denoise_profiled, "strength", 0, 0.25)
  end , "denoise strength plus quarter"
)

dt.register_event("test_diffuse", "shortcut",
  function(event, shortcut)
    for i = -1,-4, -1 do 
      enable(mods.diffuse_or_sharpen, i)
      dt.control.sleep(1000)
    end 
  end, "test diffuse"
)

dt.register_event("add_aa_sharpen", "shortcut",
  function(event, shortcut)
    dt.gui.action("iop/diffuse/preset/add local contrast", 0, "", "", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "new", 1.0)
    dt.gui.action("iop/diffuse/preset/sharpen demosaicing (AA filter)", 0, "", "", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "move down", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "move down", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "new", 1.0)
    dt.gui.action("iop/diffuse/preset/denoise: coarse", 0, "", "", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "move down", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "new", 1.0)
    dt.gui.action("iop/diffuse/preset/denoise: medium", 0, "", "", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "move down", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "new", 1.0)
    dt.gui.action("iop/diffuse/preset/denoise: fine", 0, "", "", 1.0)
    dt.gui.action("iop/diffuse", 0, "instance", "move down", 1.0)
    dt.gui.action("iop/diffuse", 4, "enable", "off", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/diffuse", 3, "enable", "off", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/diffuse", 2, "enable", "off", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/diffuse", 1, "enable", "off", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/diffuse", 0, "enable", "off", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/diffuse", 4, "enable", "on", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/diffuse", 1, "enable", "on", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/diffuse", 0, "enable", "on", 1.0)
    dt.control.sleep(1000)
    dt.gui.action("iop/denoiseprofile/preset/wavelets: chroma only", 0, "", "", 1.0)
    dt.gui.action("iop/colorbalancergb/preset/My Color", 0, "", "", 1.0)
    dt.gui.action("iop/colorbalancergb", 0, "instance", "move down", 1.0)
    dt.gui.action("iop/cacorrectrgb", 0, "", "", 1.0)
    dt.gui.action("iop/bilat", 0, "enable", "on", 1.0)
    dt.gui.action("iop/bilat", 0, "enable", "off", 1.0)
    dt.gui.action("lib/history/compress history stack", 0, "", "", 1.0)
  end, "add aa sharpen"
)